import 'package:slick/configuration/slick_configuration.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:google_fonts/google_fonts.dart';
import 'dart:math' as math;

class SlickNumberInput<T> extends StatelessWidget {
  SlickNumberInput({
    Key key,
    @required this.initialValue,
    @required this.onChanged,
    this.labelText,
    this.validator,
    this.padding = const EdgeInsets.only(
      top: 8.0,
      right: 8.0,
    ),
    this.width = 250,
    this.suffix,
    this.readOnly = false,
    this.prefix,
  }) : super(key: key);

  final T initialValue;
  final String labelText;
  final ValueChanged<T> onChanged;
  final FormFieldValidator<String> validator;
  final EdgeInsetsGeometry padding;
  final double width;
  final Widget suffix;
  final Widget prefix;
  final bool readOnly;

  @override
  Widget build(BuildContext context) {
    return Container(
      width: width,
      margin: padding,
      child: TextFormField(
        readOnly: readOnly,
        initialValue: initialValue.toString(),
        onChanged: (value) {
          var valueStringToEvaluate = value;
          if (valueStringToEvaluate.endsWith('.')) {
            valueStringToEvaluate = valueStringToEvaluate.replaceAll('.', '');
          }

          if (initialValue is int) {
            int v = int.tryParse(valueStringToEvaluate);
            if (v == null) v = 0;
            this.onChanged(v as T);
          } else if (initialValue is double) {
            double v = double.tryParse(valueStringToEvaluate);
            if (v == null) v = 0;
            this.onChanged(v as T);
          } else
            onChanged(valueStringToEvaluate as T);
        },
        keyboardType: TextInputType.numberWithOptions(
            decimal: (initialValue is int) ? false : true),
        cursorColor: SlickConfiguration.colors.inputCursor,
        style: GoogleFonts.barlow(
          fontSize: 15,
          color: SlickConfiguration.colors.input,
        ),
        decoration: InputDecoration(
          suffix: suffix,
          prefix: prefix,
          enabledBorder: UnderlineInputBorder(
            borderSide: BorderSide(
              color: SlickConfiguration.colors.inputControlEnabledBorder,
              width: 0.2,
            ),
          ),
          focusedBorder: UnderlineInputBorder(
            borderSide: BorderSide(
              color: SlickConfiguration.colors.inputControlFocusedBorder,
            ),
          ),
          border: UnderlineInputBorder(
            borderSide: BorderSide(
              color: SlickConfiguration.colors.inputControlBorder,
            ),
          ),
          floatingLabelBehavior: FloatingLabelBehavior.always,
          labelText: labelText != null ? labelText.toUpperCase() : null,
          alignLabelWithHint: true,
          hintStyle: TextStyle(
            height: 2.0,
            color: SlickConfiguration.colors.input.withAlpha(150),
          ),
          hintText: labelText != null ? labelText.toUpperCase() : null,
          labelStyle: GoogleFonts.barlow(
            fontSize: 18,
            fontWeight: FontWeight.w600,
            color: SlickConfiguration.colors.inputLabel,
          ),
          errorStyle: GoogleFonts.barlow(
            fontSize: 12,
          ),
          filled: false,
        ),
        inputFormatters: [
          DecimalTextInputFormatter(
              decimalRange: (initialValue is int) ? null : 6),
        ],
        validator: validator,
      ),
    );
  }
}

class DecimalTextInputFormatter extends TextInputFormatter {
  DecimalTextInputFormatter({this.decimalRange})
      : assert(decimalRange == null || decimalRange > 0);

  final int decimalRange;

  @override
  TextEditingValue formatEditUpdate(
    TextEditingValue oldValue, // unused.
    TextEditingValue newValue,
  ) {
    TextSelection newSelection = newValue.selection;
    String truncated = newValue.text;

    if (decimalRange != null) {
      String value = newValue.text;

      if (value.contains(".") &&
          value.substring(value.indexOf(".") + 1).length > decimalRange) {
        truncated = oldValue.text;
        newSelection = oldValue.selection;
      } else if (value == ".") {
        truncated = "0.";

        newSelection = newValue.selection.copyWith(
          baseOffset: math.min(truncated.length, truncated.length + 1),
          extentOffset: math.min(truncated.length, truncated.length + 1),
        );
      }

      return TextEditingValue(
        text: truncated,
        selection: newSelection,
        composing: TextRange.empty,
      );
    }
    return newValue;
  }
}
